import Flutter
import UIKit
import AVFoundation

public class SwiftVaisSpeechPlugin: NSObject, FlutterPlugin, AVAudioRecorderDelegate {
   // status - unset, initialized, recording, paused, stopped
      var status = "unset"
      var hasPermissions = false
      var mExtension = ""
      var mPath = ""
      var mSampleRate = 22050
      var channel = 0
      var buffer:[Int] = []
      var shouldGetBuffer = false
      var startTime: Date!
      var UNUSAGED_VALUE = 0.0
      var isRecordRealtime = false

      var audioEngine: AVAudioEngine!
      var inputNode : AVAudioInputNode!
      var outputFormat : AVAudioFormat!

    var converterNode:AVAudioMixerNode!
      var aacFileURL:NSURL!

      var outputAACFile : AVAudioFile!

      public static func register(with registrar: FlutterPluginRegistrar) {
          let channel = FlutterMethodChannel(name: "flutter_audio_recorder", binaryMessenger: registrar.messenger())
          let instance = SwiftVaisSpeechPlugin()
          registrar.addMethodCallDelegate(instance, channel: channel)
      }

      public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
          switch call.method {
          case "current":
              print("current")

              let dic = call.arguments as! [String : Any]
              channel = dic["channel"] as? Int ?? 0



              var recordingResult = [String : Any]()
              recordingResult["duration"] = getDuration()
              recordingResult["path"] = mPath
              recordingResult["audioFormat"] = mExtension
              recordingResult["peakPower"] = UNUSAGED_VALUE
              recordingResult["averagePower"] = UNUSAGED_VALUE
              recordingResult["isMeteringEnabled"] = false
              recordingResult["status"] = status
              recordingResult["iOSBuffer"] = buffer
              recordingResult["shouldGetBuffer"] = shouldGetBuffer
              shouldGetBuffer = false

              result(recordingResult)
          case "init":
              print("init")
              audioEngine = AVAudioEngine()
              inputNode = self.audioEngine.inputNode
              
              let dic = call.arguments as! [String : Any]
              mExtension = dic["extension"] as? String ?? ""
              mPath = dic["path"] as? String ?? ""
              mSampleRate = dic["sampleRate"] as? Int ?? 22050
              isRecordRealtime = dic["realtime"] as? Bool ?? false
              print("m:", mExtension, mPath)
              startTime = Date()
              if mPath == "" {
                  let documentsPath = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true)[0]
                  mPath = documentsPath + "/" + String(Int(startTime.timeIntervalSince1970)) + ".aac"
                  print("path: " + mPath)
              }
              aacFileURL = NSURL(fileURLWithPath: mPath)
              
              converterNode = AVAudioMixerNode()
              outputFormat = AVAudioFormat(commonFormat: .pcmFormatFloat32,
                                           sampleRate: Double(22050),
                                           channels: AVAudioChannelCount(1),
                                           interleaved: false)
              var settings = self.outputFormat.settings
              settings[AVFormatIDKey] = NSNumber(value: kAudioFormatMPEG4AAC)
              do {

                  outputAACFile = try AVAudioFile(forWriting: self.aacFileURL! as URL,
                                                  settings:settings)

                  let audioSession = AVAudioSession.sharedInstance()
                try audioSession.setCategory(AVAudioSession.Category.playAndRecord)
                  try audioSession.setActive(true)

                
                  self.audioEngine.attach(converterNode)
                  self.audioEngine.connect(inputNode,
                                           to: converterNode,
                                           format: inputNode.outputFormat(forBus: 0))

                  self.audioEngine.connect(converterNode,
                                           to: self.audioEngine.mainMixerNode,
                                           format: outputFormat)

                  converterNode.volume = 0

                  converterNode.installTap(onBus: 0, bufferSize: 1024, format: converterNode.outputFormat(forBus: 0)) { (buffer, when) in
                   
                    self.buffer.removeAll()
                    self.buffer.append(contentsOf: self.copyAudioBufferBytes(buffer))
                    try! self.outputAACFile.write(from: buffer)
                    self.shouldGetBuffer = true
                  }


                  status = "initialized"
                  var recordingResult = [String : Any]()
                  recordingResult["duration"] = 0
                  recordingResult["path"] = mPath
                  recordingResult["audioFormat"] = mExtension
                  recordingResult["peakPower"] = UNUSAGED_VALUE
                  recordingResult["averagePower"] = UNUSAGED_VALUE
                  recordingResult["isMeteringEnabled"] = false
                  recordingResult["status"] = status

                  result(recordingResult)
              } catch {
                  print("fail")
                  result(FlutterError(code: "", message: "Failed to init", details: error))
              }
          case "start":
              print("start")

              if status == "initialized" {
                  do {
                      try audioEngine.start()
                      status = "recording"
                  } catch {
                      result(FlutterError(code: "", message: "Failed to play", details: error))
                  }
              }

              result(nil)

          case "stop":
              print("stop")

              status = "stopped"
              var recordingResult = [String : Any]()
              recordingResult["duration"] = getDuration()
              recordingResult["path"] = mPath
              recordingResult["audioFormat"] = mExtension
              recordingResult["peakPower"] = UNUSAGED_VALUE
              recordingResult["averagePower"] = UNUSAGED_VALUE
              recordingResult["isMeteringEnabled"] = false
              recordingResult["status"] = status

              audioEngine.stop()

              result(recordingResult)
          case "pause":
              print("pause")

              if status == "recording" {
                  audioEngine.pause()
                  status = "paused"
              }

              result(nil)
          case "resume":
              print("resume")

              if status == "paused" {
                  do {
                      try audioEngine.start()
                      status = "recording"
                  } catch {
                      result(FlutterError(code: "", message: "Failed to resume", details: error))
                  }
              }

              result(nil)
          case "hasPermissions":
              print("hasPermissions")
              var permission: AVAudioSession.RecordPermission
              #if swift(>=4.2)
              permission = AVAudioSession.sharedInstance().recordPermission
              #else
              permission = AVAudioSession.sharedInstance().recordPermission()
              #endif

              switch permission {
              case .granted:
                  print("granted")
                  hasPermissions = true
                  result(hasPermissions)
                  break
              case .denied:
                  print("denied")
                  hasPermissions = false
                  result(hasPermissions)
                  break
              case .undetermined:
                  print("undetermined")

                  AVAudioSession.sharedInstance().requestRecordPermission() { [unowned self] allowed in
                      DispatchQueue.main.async {
                          if allowed {
                              self.hasPermissions = true
                              print("undetermined true")
                              result(self.hasPermissions)
                          } else {
                              self.hasPermissions = false
                              print("undetermined false")
                              result(self.hasPermissions)
                          }
                      }
                  }
                  break
              default:
                  result(hasPermissions)
                  break
              }
          default:
              result(FlutterMethodNotImplemented)
          }
      }


      // developer.apple.com/documentation/coreaudiotypes/coreaudiotype_constants/1572096-audio_data_format_identifiers
      func getOutputFormatFromString(_ format : String) -> Int {
          switch format {
          case ".mp4", ".aac", ".m4a":
              return Int(kAudioFormatMPEG4AAC)
          case ".wav":
              return Int(kAudioFormatLinearPCM)
          default :
              return Int(kAudioFormatMPEG4AAC)
          }
      }

      func getDuration() -> Int {
          do {
              let audioPlayer = try AVAudioPlayer(contentsOf: outputAACFile.url)
              print("Duration", Int(audioPlayer.duration))
              return Int(audioPlayer.duration) * 1000
          } catch {
              return -1
          }

      }
    
    
    
    func copyAudioBufferBytes(_ audioBuffer: AVAudioPCMBuffer) -> [Int] {
        let srcLeft = audioBuffer.floatChannelData![0]
      
        let arrayFloat = Array(UnsafeBufferPointer(start: srcLeft, count: Int(audioBuffer.frameLength)))

        var arrayInt:[Int] = []
        for mFloat in arrayFloat {
            arrayInt.append(Int(mFloat * 32767))
        }
        
        return arrayInt
      }
}
